import re

RE_PARAMS = re.compile('(/:([a-zA-Z_]+))')


def to_url(urltemplate, params):
    uri = None

    params_to_replace = RE_PARAMS.findall(urltemplate)

    if params_to_replace:
        for needle, key in params_to_replace:
            try:
                uri =urltemplate.replace(needle, '/%s' % params[key])
            except KeyError:
                pass
    else:
        uri = urltemplate
    return uri


def to_django_urlpattern(path):
    return RE_PARAMS.sub('/(?P<\\2>[^/]+)', path)





import functools
import logging
import mimeparse
import responses
import urltemplate
import urllib
from collections import OrderedDict

from .serializers import default_serializers
from .headers import normalize_header_name
from .context import Context


log = logging.getLogger(__name__)


def http_response(response):
    """
    RESTResponse -> HTTPResponse factory
    """

    from django.http import HttpResponse
    from .headers import parse_accept_header, build_content_type_header

    context = response.context

    content_type = None
    serializer = None
    representation = None

    if 'accept' in context.headers:
        accepting = parse_accept_header(context.headers['accept'])
        for content_type, representation, q in accepting:
            if content_type == '*/*' or content_type == 'application/*':
                content_type = 'application/json'
            if context.resource.serializers.contains(content_type)\
                and (not representation or representation in context.resource.representations):
                try:
                    representation = representation or context.resource.representations.keys()[0]
                except IndexError:
                    pass
                serializer = context.resource.serializers[content_type]
                content_type = build_content_type_header(content_type, representation)
                break
    else:
        content_type = 'application/json'

    if not content_type or not serializer:
        return HttpResponse(status=406) # Not Acceptable

    if response.data is not None:
        content = response.serialize(serializer, response.data, representation)
    else:
        content = ''

    httpresp = HttpResponse(content, status=response.status)
    httpresp['Content-Type'] = content_type

    for header, value in response.headers.items():
        httpresp[header]=value

    return httpresp


class Resource(object):
    def __init__(self, path, serializers=None):
        self._path = path
        self._callbacks = {}
        self._representations = OrderedDict()
        self._serializers = serializers or default_serializers
        # register aliases for the decorators
        for verb in ('GET', 'POST', 'PUT', 'DELETE', 'OPTIONS', 'PATCH'):
            setattr(self, verb.lower(), functools.partial(self._decorator, verb))

    def _decorator(self, method):
        def wrapper(view):
            if method in self._callbacks:
                raise ValueError('Already registered')
            self._callbacks[method] = view
            return view
        return wrapper

    @property
    def serializers(self):
        return self._serializers

    @property
    def representations(self):
        return self._representations

    def __call__(self, ctx, *args, **kw):
        method = ctx.method
        request = ctx.request

        try:
            content_length = int(request.META['CONTENT_LENGTH'])
        except (KeyError, TypeError, ValueError):
            content_length = 0

        if content_length and 'CONTENT_TYPE' in request.META:
            mimetype = mimeparse.best_match(dict(self._serializers.items()),
                    request.META['CONTENT_TYPE'])
            if mimetype:
                ctx.deserializer = self._serializers[mimetype]
                if request.body:
                    ctx.body = self._serializers[mimetype].loads(request.body)
            elif not content_length:
                self.body = None
            else:
                return http_response(ctx.NotAcceptable())

        ctx.content_type = request.META['CONTENT_TYPE']

        # prepare request headers

        headers = request.META.items()
        http_headers = dict(map(lambda x: (normalize_header_name(x[0]),x[1]),
            filter(lambda x: x[0].startswith('HTTP_'), headers)))
        ctx.headers.update(http_headers)

        log.debug('Calling %s, %s, %s' % (method, args, kw))
        if method in self._callbacks:
            resp = self._callbacks[method](ctx, *args, **kw)
            return http_response(resp)
        else:
            return http_response(ctx.MethodNotAllowed())

    def representation(self, name='default'):
        def wrapped(func):
            self._representations[name] = func
            return func
        return wrapped

    def uri(self, context, params=None, query=None):
        assert params is None or isinstance(params, dict), "entity.uri() params should be passed as dict"

        params = params or {}

        uri = context.build_absolute_uri(self._path)
        uri = urltemplate.to_url(uri, params)

        if query:
            uri += '?'+urllib.urlencode(query)

        return uri

    def convert(self, context, obj, representation=None):
        """
        Converts model (`obj`) using specified or default `representation`
        within a `context`
        """

        if representation is None:
            try:
                convert = self.representations.values()[0]
            except IndexError:
                convert = lambda x, ctx: x  # pass through
        else:
            convert = self.representations[representation]
        return convert(obj, context)




from django.utils.http import http_date
from .utils import model_to_dict
import times


class Response(object):
    def __init__(self, context, data=None, status=200, headers=None,
            last_modified=None):
        self.headers = {}
        self.headers.update(headers or {})
        self.representation = None
        self.content_type = None
        self.context = context
        self.status = status
        self.data = data
        if last_modified:
            self.set_last_modified(last_modified)

    def get_converter(self, representation):
        dummy_converter = lambda x, context: x
        converter = self.context.resource.representations.get(representation)
        return converter or dummy_converter

    def set_last_modified(self, dt):
        if dt:
            self.headers['Last-Modified'] = http_date(times.to_unix(dt))
        else:
            self.headers.pop('Last-Modified', None)

    def serialize(self, serializer, data, representation):
        if data is None:
            return ''
        else:
            if self.status >= 200 and self.status <300:
                output = self.context.resource.convert(self.context, data, representation)
            else:
                output = data
            return serializer.dumps(output)


class CreatedResponse(Response):
    def __init__(self, context, data=None, headers=None):
        super(CreatedResponse, self).__init__(context, data=data, status=201,
                headers=headers)


class NoContentResponse(Response):
    def __init__(self, context, data=None, headers=None):
        super(NoContentResponse, self).__init__(context, data=None, status=204,
                headers=headers)


class NotModifiedResponse(Response):
    def __init__(self, context, data=None, headers=None):
        super(NotModifiedResponse, self).__init__(context, data=None, status=304,
                headers=headers)


class UnauthorizedResponse(Response):
    def __init__(self, context, headers=None):
        super(UnauthorizedResponse, self).__init__(context, data=None,
                status=401, headers=headers)


class ForbiddenResponse(Response):
    def __init__(self, context, headers=None):
        super(ForbiddenResponse, self).__init__(context, data=None, status=403,
                headers=headers)


class NotFoundResponse(Response):
    def __init__(self, context, headers=None):
        super(NotFoundResponse, self).__init__(context, data=None, status=404,
                headers=headers)


class MethodNotAllowedResponse(Response):
    def __init__(self, context, headers=None):
        super(MethodNotAllowedResponse, self).__init__(context, data=None,
                status=405, headers=headers)


class CollectionResponse(Response):
    def __init__(self, context, iterable, totalCount=None, key=None, headers=None):
        super(CollectionResponse, self).__init__(context, data=iterable,
                headers=headers)
        self.key = key or 'items'
        self.totalCount = totalCount

    def serialize(self, serializer, iterable, representation):
        resp = {
                self.key: map(lambda x: self.context.resource.convert(self.context, x, representation), iterable),
                'totalCount': self.totalCount if self.totalCount is not None else len(iterable),
                }
        return serializer.dumps(resp)


class EntityResponse(Response):
    pass


class NotAcceptableResponse(Response):
    def __init__(self, context, headers=None):
        super(NotAcceptableResponse, self).__init__(context, data=None,
                status=406, headers=headers)


class ValidationErrorResponse(Response):
    def __init__(self, context, errors, headers=None):
        resp = {
                'errors': errors,
                }
        super(ValidationErrorResponse, self).__init__(context, data=resp,
                status=422, headers=headers)




import functools
from .responses import UnauthorizedResponse


def login_required(func):
    @functools.wraps(func)
    def wrapped(request, *args, **kw):
        if not request.user.is_authenticated():
            return UnauthorizedResponse(request)
        return func(request, *args, **kw)
    return wrapped




from setuptools import setup, find_packages

setup(name='restosaur',
      version='0.2',
      description='Damn simple RESTful library',
      classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Framework :: Django",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Intended Audience :: Developers",
        ],
      author='Marcin Nowak',
      author_email='marcin.j.nowak@gmail.com',
      url='https://github.com/marcinn/restosaur',
      install_requires = ['mimeparse', 'times>=0.7'],
      keywords='web rest python django',
      packages=find_packages('.'),
      include_package_data=True,
      zip_safe=True,
      )



import responses
import email
import times


def parse_http_date(header, headers):
    if header in headers and headers[header]:
        timetuple = email.utils.parsedate_tz(headers[header])
        try:
            return times.from_unix(email.utils.mktime_tz(timetuple))
        except (TypeError, ValueError):
            pass


class Context(object):
    def __init__(self, api, request, resource, method, parameters=None,
            body=None):
        self.method = method
        self.api = api
        self.headers = {}
        self.request = request
        self.body = body
        self.resource = resource
        self.parameters = parameters or {}
        self.deserializer = None
        self.content_type = None

    def build_absolute_uri(self, path):
        return self.request.build_absolute_uri('/%s%s' % (self.api.path, path))

    def is_modified_since(self, dt):
        """
        Compares datetime `dt` with `If-Modified-Since` header value.
        Returns True if `dt` is newer than `If-Modified-Since`, False otherwise.
        """
        if_modified_since = parse_http_date('if-modified-since', self.headers)

        if if_modified_since:
            return times.to_unix(dt.replace(microsecond=0)) > times.to_unix(if_modified_since)

        return True

    @property
    def deserialized(self):
        return self.body

    # response factories

    def Response(self, *args, **kwargs):
        return responses.Response(self, *args, **kwargs)

    def Created(self, *args, **kwargs):
        return responses.CreatedResponse(self, *args, **kwargs)

    def ValidationError(self, *args, **kwargs):
        return responses.ValidationErrorResponse(self, *args, **kwargs)

    def NotAcceptable(self, *args, **kwargs):
        return responses.NotAcceptableResponse(self, *args, **kwargs)

    def NotFound(self, *args, **kwargs):
        return responses.NotFoundResponse(self, *args, **kwargs)

    def NotModified(self, *args, **kwargs):
        return responses.NotModifiedResponse(self, *args, **kwargs)

    def MethodNotAllowed(self, *args, **kwargs):
        return responses.MethodNotAllowedResponse(self, *args, **kwargs)

    def Forbidden(self, *args, **kwargs):
        return responses.ForbiddenResponse(self, *args, **kwargs)

    def Unauthorized(self, *args, **kwargs):
        return responses.UnauthorizedResponse(self, *args, **kwargs)

    def NoContent(self, *args, **kwargs):
        return responses.NoContentResponse(self, *args, **kwargs)

    def Entity(self, *args, **kwargs):
        return responses.EntityResponse(self, *args, **kwargs)

    def Collection(self, *args, **kwargs):
        return responses.CollectionResponse(self, *args, **kwargs)

